(*
    Undo stack management
    Copyright (c) 2002-2003 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
unit DiffUndoBuffer;

interface

uses Classes, DiffLine;

type
  TDiffUndoItem = class
  private
    FMode: TDiffLineMode;
    FLineFrom: integer;
    FLineTo: integer;
    FFlags: integer;
    FData: TObject;
  public
    constructor Create;
    constructor CreateFrom( AMode: TDiffLineMode; AFrom, ATo, AFlags: integer );

    property Mode: TDiffLineMode read FMode write FMode;
    property LineFrom: integer read FLineFrom write FLineFrom;
    property LineTo: integer read FLineTo write FLineTo;
    property Flags: integer read FFlags write FFlags;
    property Data: TObject read FData write FData;
  end;

type
  TDiffUndoBuffer = class
  private
    FList: TList;

    function GetCount: integer;
    function GetItem( AIndex: integer ): TDiffUndoItem;
  public
    constructor Create;
    destructor Destroy; override;

    procedure Push( AItem: TDiffUndoItem );
    function Pop: TDiffUndoItem;

    procedure Clear;

    property Count: integer read GetCount;
    property Items[AIndex: integer]: TDiffUndoItem read GetItem;
  end;

implementation

constructor TDiffUndoItem.Create;
begin
  FMode := dmNone;
  FLineFrom := 0;
  FLineTo := 0;
  FData := nil;
end;

constructor TDiffUndoItem.CreateFrom( AMode: TDiffLineMode; AFrom, ATo, AFlags: integer );
begin
  FMode := AMode;
  FLineFrom := AFrom;
  FLineTo := ATo;
  FFlags := AFlags;
  FData := nil;
end;

constructor TDiffUndoBuffer.Create;
begin
  FList := TList.Create;
end;

destructor TDiffUndoBuffer.Destroy;
begin
  Clear;
  FList.Free;
  inherited Destroy;
end;

function TDiffUndoBuffer.GetCount: integer;
begin
  Result := FList.Count;
end;

procedure TDiffUndoBuffer.Push( AItem: TDiffUndoItem );
begin
  FList.Add( AItem );
end;

function TDiffUndoBuffer.Pop: TDiffUndoItem;
var
  N: integer;
begin
  Result := nil;
  N := FList.Count;
  if( N > 0 ) then begin
    Result := TDiffUndoItem( FList.Last );
    FList.Delete( N-1 );
  end;
end;

procedure TDiffUndoBuffer.Clear;
var
  I: integer;
  U: TDiffUndoItem;
begin
  for I:=0 to FList.Count-1 do begin
    U := TDiffUndoItem( FList.Items[I] );
    if( U.Data <> nil )and( U.Data is TDiffUndoBuffer ) then begin
      TDiffUndoBuffer(U.Data).Free;
    end;
    U.Free;
  end;
  FList.Clear;
end;

function TDiffUndoBuffer.GetItem( AIndex: integer ): TDiffUndoItem;
begin
  Result := nil;
  if( AIndex >= 0 )and( AIndex < FList.Count ) then begin
    Result := TDiffUndoItem( FList.Items[AIndex] );
  end;
end;

end.
