(*
    LCS - A study for a simple differencing program
    Open files window
    Copyright (c) 2002-2003 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
unit FormOpenFiles;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TOpenForm = class(TForm)
    LblLeftFile: TLabel;
    BtnBrowseLeft: TButton;
    LblRightFile: TLabel;
    BtnBrowseRight: TButton;
    OpenDlgLeft: TOpenDialog;
    OpenDlgRight: TOpenDialog;
    BtnCancel: TButton;
    BtnOk: TButton;
    CboLeft: TComboBox;
    CboRight: TComboBox;
    procedure BtnBrowseLeftClick(Sender: TObject);
    procedure BtnBrowseRightClick(Sender: TObject);
    procedure BtnOkClick(Sender: TObject);
  private
    { Private declarations }
    procedure AddToHistory( Box: TComboBox );
  public
    { Public declarations }
    LeftFile: string;
    RightFile: string;
  end;

var
  OpenForm: TOpenForm;

implementation

uses FileCtrl, ActiveX, ShlObj;

{$R *.DFM}

const
  MAX_HISTORY_LENGTH = 8;

procedure TOpenForm.AddToHistory( Box: TComboBox );
var
  I: integer;
begin
  if( Box.Text <> '' ) then begin
    I := Box.Items.IndexOf( Box.Text );
    if( I >= 0 ) then
      Box.Items.Move( I, 0 )
    else
      Box.Items.Insert( 0, Box.Text );
  end;
  if( Box.Items.Count > MAX_HISTORY_LENGTH ) then begin
    Box.Items.Delete( Box.Items.Count-1 );
  end;
end;

function ChooseFile( OD: TOpenDialog; LF: string ): boolean;
var
  D: string;
begin
  D := ExcludeTrailingBackslash( ExtractFilePath(LF) );
  if( DirectoryExists( D ) ) then
    OD.InitialDir := D;
  Result := OD.Execute;
end;

procedure TOpenForm.BtnBrowseLeftClick(Sender: TObject);
begin
  if( ChooseFile( OpenDlgLeft, CboLeft.Text ) ) then
    CboLeft.Text := OpenDlgLeft.FileName;
end;

procedure TOpenForm.BtnBrowseRightClick(Sender: TObject);
begin
  if( ChooseFile( OpenDlgRight, CboRight.Text ) ) then
    CboRight.Text := OpenDlgRight.FileName;
end;

procedure TOpenForm.BtnOkClick(Sender: TObject);
begin
  LeftFile := CboLeft.Text;
  RightFile := CboRight.Text;
  AddToHistory( CboLeft );
  AddToHistory( CboRight );
end;

{$ifdef WHOOPS_FORGOT_ABOUT_SelectDirectory}
procedure TOpenForm.BtnBrowseForDirClick(Sender: TObject);
var
  BrowseInfo: TBrowseInfo;
  SelectedPath: array[0..MAX_PATH] of char;
  PIDL: PItemIDList;
  PMalloc: IMalloc;
  D: string;
begin
  FillChar( BrowseInfo, SizeOf(BrowseInfo), #0 );
  with BrowseInfo do begin
    hwndOwner := Handle;
    pszDisplayName := SelectedPath;
    lpszTitle := 'Select default folder';
    ulFlags := BIF_RETURNONLYFSDIRS; // Only file-system folders
  end;
  PIDL := SHBrowseForFolder(BrowseInfo);
  if( PIDL <> nil ) then begin
    if SHGetPathFromIDList(PIDL,SelectedPath) then begin
      D := CboDefaultDir.Items[0];
      CboDefaultDir.Items.Delete(0);
      CboDefaultDir.Text := SelectedPath;
      AddToHistory( CboDefaultDir );
      CboDefaultDir.Items.Insert(0,D);
     end;
     if( SHGetMalloc( PMalloc ) = 0 ) then begin
       PMalloc.Free( PIDL );
       PMalloc._Release;
     end;
  end;
end;
{$endif}

end.
